@@ -30,7 +30,7 @@
  * Tests the C++ API for schema related functions.
  */
 
-#include "catch.hpp"
+#include <test/support/tdb_catch.h>
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/cpp_api/tiledb_experimental"
 #include "tiledb/sm/misc/constants.h"
@@ -653,19 +653,20 @@ TEST_CASE(
           Catch::Matchers::Equals(std::vector<uint8_t>{0, 1, 0, 0}));
       CHECK_THAT(
           d_data,
-          Catch::Matchers::Equals(std::vector<char>{'t',
-                                                    'e',
-                                                    's',
-                                                    't',
-                                                    'd',
-                                                    't',
-                                                    'e',
-                                                    's',
-                                                    't',
-                                                    't',
-                                                    'e',
-                                                    's',
-                                                    't'}));
+          Catch::Matchers::Equals(std::vector<char>{
+              't',
+              'e',
+              's',
+              't',
+              'd',
+              't',
+              'e',
+              's',
+              't',
+              't',
+              'e',
+              's',
+              't'}));
       CHECK_THAT(
           d_offsets,
           Catch::Matchers::Equals(std::vector<uint64_t>{0, 4, 5, 9}));
@@ -694,19 +695,20 @@ TEST_CASE(
           Catch::Matchers::Equals(std::vector<uint8_t>{0, 0, 0, 1}));
       CHECK_THAT(
           d_data,
-          Catch::Matchers::Equals(std::vector<char>{'t',
-                                                    'e',
-                                                    's',
-                                                    't',
-                                                    't',
-                                                    'e',
-                                                    's',
-                                                    't',
-                                                    't',
-                                                    'e',
-                                                    's',
-                                                    't',
-                                                    'd'}));
+          Catch::Matchers::Equals(std::vector<char>{
+              't',
+              'e',
+              's',
+              't',
+              't',
+              'e',
+              's',
+              't',
+              't',
+              'e',
+              's',
+              't',
+              'd'}));
       CHECK_THAT(
           d_offsets,
           Catch::Matchers::Equals(std::vector<uint64_t>{0, 4, 8, 12}));
@@ -910,22 +912,23 @@ TEST_CASE(
               0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 49, 50, 51}));
       CHECK_THAT(
           e_data,
-          Catch::Matchers::Equals(std::vector<char>{'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'n',
-                                                    'e',
-                                                    'e',
-                                                    'e',
-                                                    'e'}));
+          Catch::Matchers::Equals(std::vector<char>{
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'n',
+              'e',
+              'e',
+              'e',
+              'e'}));
       CHECK_THAT(
           e_offsets,
           Catch::Matchers::Equals(std::vector<uint64_t>{
