@@ -46,6 +46,7 @@
 #include "tiledb/sm/crypto/encryption_key.h"
 #include "tiledb/sm/fragment/fragment_info.h"
 #include "tiledb/sm/metadata/metadata.h"
+#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 using namespace tiledb::common;
 
@@ -55,7 +56,6 @@ namespace sm {
 class ArraySchema;
 class SchemaEvolution;
 class FragmentMetadata;
-class StorageManager;
 enum class QueryType : uint8_t;
 
 /**
@@ -204,6 +204,15 @@ class Array {
   /** Closes the array and frees all memory. */
   Status close();
 
+  /**
+   * Deletes the Array data with given URI.
+   *
+   * @param uri The uri of the Array whose data is to be deleted.
+   *
+   * @pre The Array must be open for exclusive writes
+   */
+  void delete_array(const URI& uri);
+
   /**
    * Deletes the fragments from the Array with given URI.
    *
