@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -51,31 +51,75 @@
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /* ********************************* */
 /*     CONSTRUCTORS & DESTRUCTORS    */
 /* ********************************* */
 
-VFS::VFS()
-    : stats_(nullptr)
-    , init_(false)
-    , read_ahead_size_(0)
-    , compute_tp_(nullptr)
-    , io_tp_(nullptr) {
-#ifdef HAVE_AZURE
-  supported_fs_.insert(Filesystem::AZURE);
-#endif
-#ifdef HAVE_GCS
-  supported_fs_.insert(Filesystem::GCS);
-#endif
+VFS::VFS(
+    stats::Stats* const parent_stats,
+    ThreadPool* const compute_tp,
+    ThreadPool* const io_tp,
+    const Config& config)
+    : stats_(parent_stats->create_child("VFS"))
+    , config_(config)
+    , compute_tp_(compute_tp)
+    , io_tp_(io_tp)
+    , vfs_params_(VFSParameters(config)) {
+  Status st;
+  assert(compute_tp);
+  assert(io_tp);
+
+  // Construct the read-ahead cache.
+  read_ahead_cache_ = tdb_unique_ptr<ReadAheadCache>(
+      tdb_new(ReadAheadCache, vfs_params_.read_ahead_cache_size_));
+
 #ifdef HAVE_HDFS
   supported_fs_.insert(Filesystem::HDFS);
+  hdfs_ = tdb_unique_ptr<hdfs::HDFS>(tdb_new(hdfs::HDFS));
+  st = hdfs_->init(config_);
+  if (!st.ok()) {
+    throw std::runtime_error("[VFS::VFS] Failed to initialize HDFS backend.");
+  }
 #endif
+
 #ifdef HAVE_S3
   supported_fs_.insert(Filesystem::S3);
+  st = s3_.init(stats_, config_, io_tp_);
+  if (!st.ok()) {
+    throw std::runtime_error("[VFS::VFS] Failed to initialize S3 backend.");
+  }
 #endif
+
+#ifdef HAVE_AZURE
+  supported_fs_.insert(Filesystem::AZURE);
+  st = azure_.init(config_, io_tp_);
+  if (!st.ok()) {
+    throw std::runtime_error("[VFS::VFS] Failed to initialize Azure backend.");
+  }
+#endif
+
+#ifdef HAVE_GCS
+  supported_fs_.insert(Filesystem::GCS);
+  st = gcs_.init(config_, io_tp_);
+  if (!st.ok()) {
+    // We should print some warning here, LOG_STATUS only prints in
+    // verbose mode. Since this is called in the init of the context, we
+    // can't return the error through the normal set it on the context.
+    throw StatusException(Status_GCSError(
+        "GCS failed to initialize, GCS support will not be available in this "
+        "context: " +
+        st.message()));
+  }
+#endif
+
+#ifdef _WIN32
+  throw_if_not_ok(win_.init(config_, io_tp_));
+#else
+  throw_if_not_ok(posix_.init(config_, io_tp_));
+#endif
+
   supported_fs_.insert(Filesystem::MEMFS);
 }
 
@@ -118,10 +162,6 @@ Config VFS::config() const {
 }
 
 Status VFS::create_dir(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot create directory; VFS not initialized"));
-
   if (!uri.is_s3() && !uri.is_azure() && !uri.is_gcs()) {
     bool is_dir;
     RETURN_NOT_OK(this->is_dir(uri, &is_dir));
@@ -176,10 +216,6 @@ Status VFS::create_dir(const URI& uri) const {
 }
 
 Status VFS::dir_size(const URI& dir_name, uint64_t* dir_size) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot get directory size; VFS not initialized"));
-
   // Sanity check
   bool is_dir;
   RETURN_NOT_OK(this->is_dir(dir_name, &is_dir));
@@ -211,10 +247,6 @@ Status VFS::dir_size(const URI& dir_name, uint64_t* dir_size) const {
 }
 
 Status VFS::touch(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot touch file; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.touch(uri.to_path());
@@ -259,19 +291,11 @@ Status VFS::touch(const URI& uri) const {
 }
 
 Status VFS::cancel_all_tasks() {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot cancel all tasks; VFS not initialized"));
-
   cancelable_tasks_.cancel_all_tasks();
   return Status::Ok();
 }
 
 Status VFS::create_bucket(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot create bucket; VFS not initialized"));
-
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     return s3_.create_bucket(uri);
@@ -302,10 +326,6 @@ Status VFS::create_bucket(const URI& uri) const {
 }
 
 Status VFS::remove_bucket(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot remove bucket; VFS not initialized"));
-
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     return s3_.remove_bucket(uri);
@@ -336,11 +356,6 @@ Status VFS::remove_bucket(const URI& uri) const {
 }
 
 Status VFS::empty_bucket(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot empty bucket; VFS not "
-                        "initialized"));
-
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     return s3_.empty_bucket(uri);
@@ -371,11 +386,6 @@ Status VFS::empty_bucket(const URI& uri) const {
 }
 
 Status VFS::is_empty_bucket(const URI& uri, bool* is_empty) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot check if bucket is empty; "
-                        "VFS not initialized"));
-
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     return s3_.is_empty_bucket(uri, is_empty);
@@ -409,11 +419,6 @@ Status VFS::is_empty_bucket(const URI& uri, bool* is_empty) const {
 }
 
 Status VFS::remove_dir(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot remove directory; VFS not "
-                        "initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.remove_dir(uri.to_path());
@@ -454,10 +459,6 @@ Status VFS::remove_dir(const URI& uri) const {
 }
 
 Status VFS::remove_file(const URI& uri) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot remove file; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.remove_file(uri.to_path());
@@ -502,10 +503,6 @@ Status VFS::remove_file(const URI& uri) const {
 }
 
 Status VFS::max_parallel_ops(const URI& uri, uint64_t* ops) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot get max parallel ops; VFS not initialized"));
-
   bool found;
   *ops = 0;
 
@@ -537,10 +534,6 @@ Status VFS::max_parallel_ops(const URI& uri, uint64_t* ops) const {
 
 Status VFS::file_size(const URI& uri, uint64_t* size) const {
   stats_->add_counter("file_size_num", 1);
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot get file size; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.file_size(uri.to_path(), size);
@@ -585,10 +578,6 @@ Status VFS::file_size(const URI& uri, uint64_t* size) const {
 }
 
 Status VFS::is_dir(const URI& uri, bool* is_dir) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot check directory; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     *is_dir = win_.is_dir(uri.to_path());
@@ -639,13 +628,7 @@ Status VFS::is_dir(const URI& uri, bool* is_dir) const {
 }
 
 Status VFS::is_file(const URI& uri, bool* is_file) const {
-  if (!init_) {
-    return LOG_STATUS(
-        Status_VFSError("Cannot check file; VFS not initialized"));
-  }
-
   stats_->add_counter("is_object_num", 1);
-
   if (uri.is_file()) {
 #ifdef _WIN32
     *is_file = win_.is_file(uri.to_path());
@@ -697,10 +680,6 @@ Status VFS::is_file(const URI& uri, bool* is_file) const {
 }
 
 Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot check bucket; VFS not initialized"));
-
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     RETURN_NOT_OK(s3_.is_bucket(uri, is_bucket));
@@ -734,84 +713,6 @@ Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
       Status_VFSError("Unsupported URI scheme: " + uri.to_string()));
 }
 
-Status VFS::init(
-    stats::Stats* const parent_stats,
-    ThreadPool* const compute_tp,
-    ThreadPool* const io_tp,
-    const Config* const ctx_config,
-    const Config* const vfs_config) {
-  stats_ = parent_stats->create_child("VFS");
-
-  assert(compute_tp);
-  assert(io_tp);
-  compute_tp_ = compute_tp;
-  io_tp_ = io_tp;
-
-  // Set appropriately the config
-  if (ctx_config)
-    config_ = *ctx_config;
-  if (vfs_config)
-    config_.inherit(*vfs_config);
-
-  // Construct the read-ahead cache.
-  bool found = false;
-  uint64_t read_ahead_cache_size = 0;
-  RETURN_NOT_OK(config_.get<uint64_t>(
-      "vfs.read_ahead_cache_size", &read_ahead_cache_size, &found));
-  assert(found);
-  read_ahead_cache_ = tdb_unique_ptr<ReadAheadCache>(
-      tdb_new(ReadAheadCache, read_ahead_cache_size));
-
-  // Store the read-ahead size.
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("vfs.read_ahead_size", &read_ahead_size_, &found));
-  assert(found);
-
-#ifdef HAVE_HDFS
-  hdfs_ = tdb_unique_ptr<hdfs::HDFS>(tdb_new(hdfs::HDFS));
-  RETURN_NOT_OK(hdfs_->init(config_));
-#endif
-
-#ifdef HAVE_S3
-  RETURN_NOT_OK(s3_.init(stats_, config_, io_tp_));
-#endif
-
-#ifdef HAVE_AZURE
-  RETURN_NOT_OK(azure_.init(config_, io_tp_));
-#endif
-
-#ifdef HAVE_GCS
-  Status st = gcs_.init(config_, io_tp_);
-  if (!st.ok()) {
-    // We should print some warning here, this LOG_STATUS only prints in
-    // verbose mode. Since this is called in the init of the context, we
-    // can't return the error through the normal set it on the context.
-    LOG_STATUS(Status_GCSError(
-        "GCS failed to initialize, GCS support will not be available in this "
-        "context: " +
-        st.message()));
-  }
-#endif
-
-#ifdef _WIN32
-  win_.init(config_, io_tp_);
-#else
-  posix_.init(config_, io_tp_);
-#endif
-
-  init_ = true;
-
-  return Status::Ok();
-}
-
-Status VFS::terminate() {
-#ifdef HAVE_S3
-  return s3_.disconnect();
-#endif
-
-  return Status::Ok();
-}
-
 Status VFS::ls(const URI& parent, std::vector<URI>* uris) const {
   stats_->add_counter("ls_num", 1);
   auto&& [st, entries] = ls_with_sizes(parent);
@@ -826,11 +727,6 @@ Status VFS::ls(const URI& parent, std::vector<URI>* uris) const {
 
 tuple<Status, optional<std::vector<directory_entry>>> VFS::ls_with_sizes(
     const URI& parent) const {
-  if (!init_) {
-    auto st = LOG_STATUS(Status_VFSError("Cannot list; VFS not initialized"));
-    return {st, std::nullopt};
-  }
-
   // Noop if `parent` is not a directory, do not error out.
   // For S3, GCS and Azure, `ls` on a non-directory will just
   // return an empty `uris` vector.
@@ -911,9 +807,6 @@ tuple<Status, optional<std::vector<directory_entry>>> VFS::ls_with_sizes(
 }
 
 Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot move file; VFS not initialized"));
-
   // If new_uri exists, delete it or raise an error based on `force`
   bool is_file;
   RETURN_NOT_OK(this->is_file(new_uri, &is_file));
@@ -1000,10 +893,6 @@ Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot move directory; VFS not initialized"));
-
   // File
   if (old_uri.is_file()) {
     if (new_uri.is_file()) {
@@ -1084,9 +973,6 @@ Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot copy file; VFS not initialized"));
-
   // If new_uri exists, delete it or raise an error based on `force`
   bool is_file;
   RETURN_NOT_OK(this->is_file(new_uri, &is_file));
@@ -1168,10 +1054,6 @@ Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::copy_dir(const URI& old_uri, const URI& new_uri) {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot copy directory; VFS not initialized"));
-
   // File
   if (old_uri.is_file()) {
     if (new_uri.is_file()) {
@@ -1254,15 +1136,8 @@ Status VFS::read(
     bool use_read_ahead) {
   stats_->add_counter("read_byte_num", nbytes);
 
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot read; VFS not initialized"));
-
   // Get config params
-  bool found;
-  uint64_t min_parallel_size = 0;
-  RETURN_NOT_OK(config_.get<uint64_t>(
-      "vfs.min_parallel_size", &min_parallel_size, &found));
-  assert(found);
+  uint64_t min_parallel_size = vfs_params_.min_parallel_size_;
   uint64_t max_ops = 0;
   RETURN_NOT_OK(max_parallel_ops(uri, &max_ops));
 
@@ -1425,7 +1300,7 @@ Status VFS::read_ahead_impl(
   //    to a future small read.
   // 3. It saves us a copy. We must make a copy of the buffer at
   //    some point (one for the user, one for the cache).
-  if (nbytes >= read_ahead_size_)
+  if (nbytes >= vfs_params_.read_ahead_size_)
     return read_fn(uri, offset, buffer, nbytes, 0, &nbytes_read);
 
   // Avoid a read if the requested buffer can be read from the
@@ -1442,11 +1317,11 @@ Status VFS::read_ahead_impl(
   // the subrange of this buffer back to the user to satisfy the
   // read request.
   Buffer ra_buffer;
-  RETURN_NOT_OK(ra_buffer.realloc(read_ahead_size_));
+  RETURN_NOT_OK(ra_buffer.realloc(vfs_params_.read_ahead_size_));
 
   // Calculate the exact number of bytes to populate `ra_buffer`
-  // with `read_ahead_size_` bytes.
-  const uint64_t ra_nbytes = read_ahead_size_ - nbytes;
+  // with `vfs_params_.read_ahead_size_` bytes.
+  const uint64_t ra_nbytes = vfs_params_.read_ahead_size_ - nbytes;
 
   // Read into `ra_buffer`.
   RETURN_NOT_OK(
@@ -1469,10 +1344,6 @@ Status VFS::read_all(
     ThreadPool* thread_pool,
     std::vector<ThreadPool::Task>* tasks,
     const bool use_read_ahead) {
-  if (!init_) {
-    return LOG_STATUS(Status_VFSError("Cannot read all; VFS not initialized"));
-  }
-
   if (regions.empty()) {
     return Status::Ok();
   }
@@ -1519,10 +1390,6 @@ Status VFS::read_all_no_batching(
     ThreadPool* thread_pool,
     std::vector<ThreadPool::Task>* tasks,
     const bool use_read_ahead) {
-  if (!init_) {
-    return LOG_STATUS(Status_VFSError("Cannot read all; VFS not initialized"));
-  }
-
   if (regions.empty()) {
     return Status::Ok();
   }
@@ -1551,21 +1418,6 @@ Status VFS::read_all_no_batching(
 Status VFS::compute_read_batches(
     const std::vector<tuple<uint64_t, Tile*, uint64_t>>& regions,
     std::vector<BatchedRead>* batches) const {
-  // Get config params
-  bool found;
-  uint64_t max_batch_size = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("vfs.max_batch_size", &max_batch_size, &found));
-  assert(found);
-  uint64_t min_batch_size = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("vfs.min_batch_size", &min_batch_size, &found));
-  assert(found);
-  uint64_t min_batch_gap = 0;
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("vfs.min_batch_gap", &min_batch_gap, &found));
-  assert(found);
-
   // Ensure the regions are sorted on offset.
   std::vector<tuple<uint64_t, Tile*, uint64_t>> sorted_regions(
       regions.begin(), regions.end());
@@ -1586,8 +1438,9 @@ Status VFS::compute_read_batches(
     uint64_t nbytes = std::get<2>(region);
     uint64_t new_batch_size = (offset + nbytes) - curr_batch.offset;
     uint64_t gap = offset - (curr_batch.offset + curr_batch.nbytes);
-    if (new_batch_size <= max_batch_size &&
-        (new_batch_size <= min_batch_size || gap <= min_batch_gap)) {
+    if (new_batch_size <= vfs_params_.max_batch_size_ &&
+        (new_batch_size <= vfs_params_.min_batch_size_ ||
+         gap <= vfs_params_.min_batch_gap_)) {
       // Extend current batch.
       curr_batch.nbytes = new_batch_size;
       curr_batch.regions.push_back(region);
@@ -1626,9 +1479,6 @@ bool VFS::supports_uri_scheme(const URI& uri) const {
 }
 
 Status VFS::sync(const URI& uri) {
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot sync; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
@@ -1673,9 +1523,6 @@ Status VFS::sync(const URI& uri) {
 }
 
 Status VFS::open_file(const URI& uri, VFSMode mode) {
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot open file; VFS not initialized"));
-
   bool is_file;
   RETURN_NOT_OK(this->is_file(uri, &is_file));
 
@@ -1728,10 +1575,6 @@ Status VFS::open_file(const URI& uri, VFSMode mode) {
 }
 
 Status VFS::close_file(const URI& uri) {
-  if (!init_)
-    return LOG_STATUS(
-        Status_VFSError("Cannot close file; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
@@ -1779,9 +1622,6 @@ Status VFS::write(const URI& uri, const void* buffer, uint64_t buffer_size) {
   stats_->add_counter("write_byte_num", buffer_size);
   stats_->add_counter("write_ops_num", 1);
 
-  if (!init_)
-    return LOG_STATUS(Status_VFSError("Cannot write; VFS not initialized"));
-
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.write(uri.to_path(), buffer, buffer_size);
@@ -1926,5 +1766,4 @@ Status VFS::flush_multipart_file_buffer(const URI& uri) {
   return Status::Ok();
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
