@@ -132,7 +132,7 @@ Status FloatScalingFilter::run_forward(
 
 Status FloatScalingFilter::run_forward(
     const Tile& tile,
-    Tile* const,  // offsets_tile
+    void* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -227,7 +227,7 @@ Status FloatScalingFilter::run_reverse(
 
 Status FloatScalingFilter::run_reverse(
     const Tile& tile,
-    Tile* const,  // offsets_tile
+    void*,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
