@@ -167,41 +167,49 @@ void ResultTile::erase_tile(const std::string& name) {
 }
 
 void ResultTile::init_attr_tile(
-    const std::string& name, bool var_size, bool nullable) {
-  // Nothing to do for the special zipped coordinates tile
+    const format_version_t format_version,
+    const ArraySchema& array_schema,
+    const std::string& name,
+    const TileSizes tile_sizes) {
+  auto tuple = TileTuple(format_version, array_schema, name, tile_sizes);
+
   if (name == constants::coords) {
-    coords_tile_ = TileTuple(false, false);
+    coords_tile_ = std::move(tuple);
     return;
   }
 
   if (name == constants::timestamps) {
-    timestamps_tile_ = TileTuple(false, false);
+    timestamps_tile_ = std::move(tuple);
     return;
   }
 
   if (name == constants::delete_timestamps) {
-    delete_timestamps_tile_ = TileTuple(false, false);
+    delete_timestamps_tile_ = std::move(tuple);
     return;
   }
 
   if (name == constants::delete_condition_index) {
-    delete_condition_index_tile_ = TileTuple(false, false);
+    delete_condition_index_tile_ = std::move(tuple);
     return;
   }
 
   // Handle attributes
   for (auto& at : attr_tiles_) {
     if (at.first == name && at.second == nullopt) {
-      at.second = TileTuple(var_size, nullable);
+      at.second = std::move(tuple);
       return;
     }
   }
 }
 
 void ResultTile::init_coord_tile(
-    const std::string& name, bool var_size, unsigned dim_idx) {
-  coord_tiles_[dim_idx] =
-      std::pair<std::string, TileTuple>(name, TileTuple(var_size, false));
+    const format_version_t format_version,
+    const ArraySchema& array_schema,
+    const std::string& name,
+    const TileSizes tile_sizes,
+    unsigned dim_idx) {
+  coord_tiles_[dim_idx] = std::pair<std::string, TileTuple>(
+      name, TileTuple(format_version, array_schema, name, tile_sizes));
 
   // When at least one unzipped coordinate has been initialized, we will
   // use the unzipped `coord()` implementation.
