@@ -87,7 +87,7 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
           subarray,
           layout,
           condition) {
-  include_coords_ = subarray_.is_set() || bitsort_attribute_.has_value();
+  include_coords_ = false;
   SparseIndexReaderBase::init(skip_checks_serialization);
 
   // Initialize memory budget variables.
@@ -158,6 +158,9 @@ Status SparseUnorderedWithDupsReader<BitmapType>::initialize_memory_budget() {
 
 template <class BitmapType>
 Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
+  // Subarray is not known to be explicitly set until buffers are deserialized
+  include_coords_ = subarray_.is_set() || bitsort_attribute_.has_value();
+
   auto timer_se = stats_->start_timer("dowork");
 
   // Make sure user didn't request delete timestamps.
@@ -215,7 +218,6 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
     for (auto& result_tile : result_tiles_) {
       if (!result_tile.coords_loaded()) {
         result_tile.set_coords_loaded();
-        ;
         result_tiles_created.emplace_back(&result_tile);
       } else {
         result_tiles_loaded.emplace_back(&result_tile);
@@ -1802,4 +1804,4 @@ template SparseUnorderedWithDupsReader<uint64_t>::SparseUnorderedWithDupsReader(
     bool);
 
 }  // namespace sm
-}  // namespace tiledb
\ No newline at end of file
+}  // namespace tiledb
