@@ -241,7 +241,7 @@ Status array_open_to_capnp(
   // Set config
   auto config_builder = array_open_builder->initConfig();
   auto config = array.config();
-  RETURN_NOT_OK(config_to_capnp(&config, &config_builder));
+  RETURN_NOT_OK(config_to_capnp(config, &config_builder));
 
   array_open_builder->setQueryType(query_type_str(array.get_query_type()));
 
