@@ -51,6 +51,7 @@
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/filter/bit_width_reduction_filter.h"
 #include "tiledb/sm/filter/bitshuffle_filter.h"
+#include "tiledb/sm/filter/bitsort_filter.h"
 #include "tiledb/sm/filter/byteshuffle_filter.h"
 #include "tiledb/sm/filter/checksum_md5_filter.h"
 #include "tiledb/sm/filter/checksum_sha256_filter.h"
@@ -60,6 +61,7 @@
 #include "tiledb/sm/filter/float_scaling_filter.h"
 #include "tiledb/sm/filter/noop_filter.h"
 #include "tiledb/sm/filter/positive_delta_filter.h"
+#include "tiledb/sm/filter/webp_filter.h"
 #include "tiledb/sm/filter/xor_filter.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/serialization/array_schema.h"
@@ -132,6 +134,8 @@ Status filter_to_capnp(
     case FilterType::FILTER_CHECKSUM_SHA256:
     case FilterType::INTERNAL_FILTER_AES_256_GCM:
     case FilterType::FILTER_XOR:
+    case FilterType::FILTER_BITSORT:
+    case FilterType::FILTER_WEBP:
       break;
   }
 
@@ -153,7 +157,7 @@ Status filter_pipeline_to_capnp(
   for (unsigned i = 0; i < num_filters; i++) {
     const auto* filter = filter_pipeline->get_filter(i);
     auto filter_builder = filter_list_builder[i];
-    filter_to_capnp(filter, &filter_builder);
+    throw_if_not_ok(filter_to_capnp(filter, &filter_builder));
   }
 
   return Status::Ok();
@@ -232,6 +236,17 @@ tuple<Status, optional<shared_ptr<Filter>>> filter_from_capnp(
     case FilterType::FILTER_XOR: {
       return {Status::Ok(), tiledb::common::make_shared<XORFilter>(HERE())};
     }
+    case FilterType::FILTER_BITSORT: {
+      return {Status::Ok(), tiledb::common::make_shared<BitSortFilter>(HERE())};
+    }
+    case FilterType::FILTER_WEBP: {
+      if constexpr (webp_filter_exists) {
+        return {Status::Ok(), tiledb::common::make_shared<WebpFilter>(HERE())};
+      } else {
+        throw std::logic_error(
+            "Can't create WebP filter; built with TILEDB_WEBP=OFF");
+      }
+    }
     default: {
       throw std::logic_error(
           "Invalid data received from filter pipeline capnp reader, unknown "
@@ -783,7 +798,7 @@ ArraySchema array_schema_from_capnp(
   // This would have been a list of size 3, so only set the version
   // if the list size is 1, meaning tiledb 1.8 or later
   // #TODO Add security validation
-  uint32_t version = constants::format_version;
+  format_version_t version = constants::format_version;
   if (schema_reader.hasVersion() && schema_reader.getVersion().size() == 1) {
     version = schema_reader.getVersion()[0];
   }
