@@ -86,8 +86,8 @@ SubarrayPartitioner::SubarrayPartitioner(
   state_.end_ = (range_num > 0) ? range_num - 1 : 0;
 
   bool found = false;
-  config_->get<bool>(
-      "sm.skip_est_size_partitioning", &skip_split_on_est_size_, &found);
+  throw_if_not_ok(config_->get<bool>(
+      "sm.skip_est_size_partitioning", &skip_split_on_est_size_, &found));
   (void)found;
   assert(found);
 }
@@ -592,7 +592,7 @@ Status SubarrayPartitioner::split_current(bool* unsplittable) {
     if (state_.multi_range_.empty())
       state_.start_ = current_.start_;
     state_.multi_range_.push_front(current_.partition_);
-    split_top_multi_range(unsplittable);
+    throw_if_not_ok(split_top_multi_range(unsplittable));
     return next_from_multi_range(unsplittable);
   }
 
@@ -618,7 +618,7 @@ Status SubarrayPartitioner::split_current(bool* unsplittable) {
       if (state_.multi_range_.empty())
         state_.start_ = current_.start_;
       state_.multi_range_.push_front(current_.partition_);
-      split_top_multi_range(unsplittable);
+      throw_if_not_ok(split_top_multi_range(unsplittable));
       return next_from_multi_range(unsplittable);
     }
 
@@ -633,7 +633,7 @@ Status SubarrayPartitioner::split_current(bool* unsplittable) {
   if (state_.single_range_.empty())
     state_.start_--;
   state_.single_range_.push_front(current_.partition_);
-  split_top_single_range(unsplittable);
+  throw_if_not_ok(split_top_single_range(unsplittable));
   return next_from_single_range(unsplittable);
 }
 
@@ -913,7 +913,7 @@ void SubarrayPartitioner::compute_splitting_value_on_tiles(
   const Range* r;
   for (auto d : dims) {
     auto dim{array_schema.domain().dimension_ptr(d)};
-    range.get_range(d, 0, &r);
+    throw_if_not_ok(range.get_range(d, 0, &r));
     auto tiles_apart = dim->tile_num(*r) - 1;
     if (tiles_apart != 0) {
       *splitting_dim = d;
@@ -986,7 +986,7 @@ void SubarrayPartitioner::compute_splitting_value_single_range(
   const Range* r;
   for (auto d : dims) {
     auto dim{array_schema.dimension_ptr(d)};
-    range.get_range(d, 0, &r);
+    throw_if_not_ok(range.get_range(d, 0, &r));
     if (!r->unary()) {
       *splitting_dim = d;
       dim->splitting_value(*r, splitting_value, unsplittable);
@@ -1031,7 +1031,7 @@ void SubarrayPartitioner::compute_splitting_value_single_range_hilbert(
   // Check for unsplittable again
   auto dim{array_schema.dimension_ptr(*splitting_dim)};
   const Range* r;
-  range.get_range(*splitting_dim, 0, &r);
+  throw_if_not_ok(range.get_range(*splitting_dim, 0, &r));
   if (dim->smaller_than(*splitting_value, *r)) {
     *unsplittable = true;
     return;
@@ -1102,7 +1102,7 @@ Status SubarrayPartitioner::compute_splitting_value_multi_range(
     }
 
     // Check if we need to split single range
-    partition.get_range(d, 0, &r);
+    throw_if_not_ok(partition.get_range(d, 0, &r));
     auto dim{array_schema.dimension_ptr(d)};
     if (!r->unary()) {
       *splitting_dim = d;
@@ -1141,34 +1141,38 @@ bool SubarrayPartitioner::must_split(Subarray* partition) {
     // Compute max memory sizes
     if (var_size) {
       if (!nullable) {
-        partition->get_max_memory_size(
+        throw_if_not_ok(partition->get_max_memory_size(
             b.first.c_str(),
             &mem_size_fixed,
             &mem_size_var,
             config_,
-            compute_tp_);
+            compute_tp_));
       } else {
-        partition->get_max_memory_size_nullable(
+        throw_if_not_ok(partition->get_max_memory_size_nullable(
             b.first.c_str(),
             &mem_size_fixed,
             &mem_size_var,
             &mem_size_validity,
             config_,
-            compute_tp_);
+            compute_tp_));
       }
     } else {
       if (!nullable) {
-        partition->get_max_memory_size(
-            b.first.c_str(), &mem_size_fixed, config_, compute_tp_);
+        throw_if_not_ok(partition->get_max_memory_size(
+            b.first.c_str(), &mem_size_fixed, config_, compute_tp_));
       } else {
-        partition->get_est_result_size_nullable(
-            b.first.c_str(), &size_fixed, &size_validity, config_, compute_tp_);
-        partition->get_max_memory_size_nullable(
+        throw_if_not_ok(partition->get_est_result_size_nullable(
+            b.first.c_str(),
+            &size_fixed,
+            &size_validity,
+            config_,
+            compute_tp_));
+        throw_if_not_ok(partition->get_max_memory_size_nullable(
             b.first.c_str(),
             &mem_size_fixed,
             &mem_size_validity,
             config_,
-            compute_tp_);
+            compute_tp_));
       }
     }
 
@@ -1176,28 +1180,28 @@ bool SubarrayPartitioner::must_split(Subarray* partition) {
     if (!skip_split_on_est_size_) {
       if (var_size) {
         if (!nullable) {
-          partition->get_est_result_size(
-              b.first.c_str(), &size_fixed, &size_var, config_, compute_tp_);
+          throw_if_not_ok(partition->get_est_result_size(
+              b.first.c_str(), &size_fixed, &size_var, config_, compute_tp_));
         } else {
-          partition->get_est_result_size_nullable(
+          throw_if_not_ok(partition->get_est_result_size_nullable(
               b.first.c_str(),
               &size_fixed,
               &size_var,
               &size_validity,
               config_,
-              compute_tp_);
+              compute_tp_));
         }
       } else {
         if (!nullable) {
-          partition->get_est_result_size_internal(
-              b.first.c_str(), &size_fixed, config_, compute_tp_);
+          throw_if_not_ok(partition->get_est_result_size_internal(
+              b.first.c_str(), &size_fixed, config_, compute_tp_));
         } else {
-          partition->get_est_result_size_nullable(
+          throw_if_not_ok(partition->get_est_result_size_nullable(
               b.first.c_str(),
               &size_fixed,
               &size_validity,
               config_,
-              compute_tp_);
+              compute_tp_));
         }
       }
     }
@@ -1221,7 +1225,7 @@ Status SubarrayPartitioner::next_from_multi_range(bool* unsplittable) {
   if (state_.multi_range_.empty()) {
     auto s = subarray_.get_subarray(current_.start_, current_.end_);
     state_.multi_range_.push_front(std::move(s));
-    split_top_multi_range(unsplittable);
+    throw_if_not_ok(split_top_multi_range(unsplittable));
   }
 
   // Loop until you find a partition that fits or unsplittable
@@ -1250,7 +1254,7 @@ Status SubarrayPartitioner::next_from_single_range(bool* unsplittable) {
   if (state_.single_range_.empty()) {
     auto s = subarray_.get_subarray(current_.start_, current_.end_);
     state_.single_range_.push_front(std::move(s));
-    split_top_single_range(unsplittable);
+    throw_if_not_ok(split_top_single_range(unsplittable));
   }
 
   // Loop until you find a partition that fits or unsplittable
@@ -1392,7 +1396,7 @@ void SubarrayPartitioner::compute_range_uint64(
   for (uint32_t d = 0; d < dim_num; ++d) {
     auto dim{array_schema.dimension_ptr(d)};
     auto var = dim->var_size();
-    range.get_range(d, 0, &r);
+    throw_if_not_ok(range.get_range(d, 0, &r));
     empty_start = var ? (r->start_size() == 0) : r->empty();
     empty_end = var ? (r->end_size() == 0) : r->empty();
     auto max_default =
